// MicroPauseWindow.hh --- window for the micropause
//
// Copyright (C) 2001, 2002 Rob Caelers <robc@krandor.org>
// All rights reserved.
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// $Id: MicroPauseWindow.hh,v 1.1.1.1 2002/09/17 22:30:04 rcaelers Exp $
//

#ifndef MICROPAUSEWINDOW_HH
#define MICROPAUSEWINDOW_HH

#include <stdio.h>

#include "BreakInterface.hh"
#include "BreakWindow.hh"
#include "BreakWindowInterface.hh"

class TimerInterface;
class ControlInterface;
class TimeBar;
class Frame;

class MicroPauseWindow :
  public BreakWindow,
  public BreakWindowInterface
{
public:
  MicroPauseWindow(TimerInterface *timer, bool ignorable);
  ~MicroPauseWindow();

  void start();
  void stop();
  void destroy();
  void set_progress(int value, int max_value);
  void set_insist_break(bool insist);
  void heartbeat();
  
protected:
  void on_postpone_button_clicked();
  void on_skip_button_clicked();
  
private:
  void refresh();
  void refresh_time_bar();
  void refresh_label();
  
private:
  //!
  TimerInterface *restbreak_timer;
  
  //! Time bar
  TimeBar *time_bar;

  // Label
  Gtk::Label *label;

  //!
  int progress_value;

  //!
  int progress_max_value;

  //!
  bool insist_break;
};

#endif // MICROPAUSEWINDOW_HH
